#pragma once

#include <utility>
#include <Qt>
#include <QObject>
#include <QWidget>
#include <QMutex>
#include <QPainter>
#include <QBrush>
#include <QApplication>
#include <QPalette>
#include <QStyle>
#include <bgtu/computer_graphics_lab_work/utilities/default_renderer_linear.hpp>
#include "owned_qimage.hpp"
#include "separate_threaded_renderer.hpp"
#include "_renderer_widget.hpp"


namespace BGTU::ComputerGraphicsLabWork::QtUtilities {
	template<class sprite_data_t>
	class RendererWidget : public _RendererWidget_Slots {
	private:
		QMutex sync;
		OwnedQImage<void> next_image;
		OwnedQImage<void> current_image;
		_SeparateThreadedRenderer_Signals *renderer;

	private slots:

		void receive_image(OwnedQImage<void> img) {
			this->sync.lock();
			std::swap(this->next_image, img);
			this->sync.unlock();
			this->update();
		}


	public:
		explicit RendererWidget(_SeparateThreadedRenderer_Signals *renderer, QWidget *owner = nullptr) : _RendererWidget_Slots{owner}, sync{}, next_image{}, current_image{}, renderer{renderer} {
			connect(this->renderer, &_SeparateThreadedRenderer_Signals::frame_rendered, this, &_RendererWidget_Slots::receive_image);
		}

	protected:
		void paintEvent(QPaintEvent *event) final {
			this->sync.lock();
			this->current_image = std::move(this->next_image);
			this->sync.unlock();

			QPainter painter;
			painter.begin(this);
			painter.setBrush(QApplication::style()->standardPalette().brush(QPalette::Background));
			painter.drawRect(this->rect());
			painter.drawImage(this->current_image.rect(), this->current_image);
			painter.end();
		}
	};
}